"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getPolkadotApi = void 0;
const api_1 = require("@polkadot/api");
const lru_cache_1 = __importDefault(require("lru-cache"));
const tenMin = 10 * 60 * 1000;
const cache = new lru_cache_1.default({
    max: 20,
    ttl: tenMin,
    updateAgeOnGet: true,
    ttlAutopurge: true,
    dispose: async (promise) => {
        const api = await promise;
        if (api.isConnected) {
            api.disconnect();
        }
    },
});
async function getPolkadotApi(ws) {
    const promise = cache.get(ws) ||
        api_1.ApiPromise.create({
            provider: new api_1.WsProvider(ws),
        });
    cache.set(ws, promise);
    const api = await promise;
    await api.isReady;
    return api;
}
exports.getPolkadotApi = getPolkadotApi;
//# sourceMappingURL=polkadot.api.js.map