import '@moonbeam-network/api-augment';
import { Asset, AssetSymbol, BalanceConfig, ChainKey, ExtrinsicConfig, MinBalanceConfig, MoonChain, XcmConfigBuilder } from '@moonbeam-network/xcm-config';
import { ApiPromise } from '@polkadot/api';
import { SubmittableExtrinsic, UnsubscribePromise } from '@polkadot/api/types';
import { AccountInfo } from '@polkadot/types/interfaces';
import { MoonbeamRuntimeXcmConfigAssetType, PalletAssetsAssetAccount, PalletAssetsAssetMetadata } from '@polkadot/types/lookup';
import { AssetBalanceInfo } from './polkadot.interfaces';
export declare class PolkadotService<Symbols extends AssetSymbol = AssetSymbol, ChainKeys extends ChainKey = ChainKey> {
    #private;
    constructor(api: ApiPromise);
    static create<Symbols extends AssetSymbol = AssetSymbol, ChainKeys extends ChainKey = ChainKey>(ws: string): Promise<PolkadotService<Symbols, ChainKeys>>;
    getMetadata(): {
        decimals: number;
        symbol: Symbols;
    };
    getExistentialDeposit(): bigint;
    getAssetDecimals(asset: Asset<Symbols>): Promise<number>;
    getAssetMeta(asset: Asset<Symbols>): Promise<PalletAssetsAssetMetadata>;
    subscribeToAccountInfo(account: string, callback: (info: AccountInfo) => void): UnsubscribePromise;
    subscribeToBalance(account: string, callback: (balance: bigint) => void): UnsubscribePromise;
    getGenericBalance(account: string, { pallet, function: fn, getParams, path, calc }: BalanceConfig<Symbols>): Promise<bigint>;
    getAssetMinBalance({ pallet, function: fn, params, path, }: MinBalanceConfig): Promise<bigint>;
    getXcmExtrinsic(account: string, amount: bigint, { pallet, extrinsic, getParams }: ExtrinsicConfig<Symbols>, fee?: bigint, primaryAccount?: string): SubmittableExtrinsic<'promise'>;
    getAssetFee(asset: Asset, weight: number, moonChain: MoonChain): Promise<bigint>;
    getUnitsPerSecond(asset: Asset, moonChain: MoonChain): Promise<bigint>;
    getAssetType(id: string): Promise<MoonbeamRuntimeXcmConfigAssetType | undefined>;
    subscribeToAssetsBalanceInfo(account: string, config: XcmConfigBuilder<Symbols, ChainKeys>, callback: (data: AssetBalanceInfo<Symbols>[]) => void): UnsubscribePromise;
    getAssetsMetadata(assetsArray: Asset<Symbols>[]): Promise<PalletAssetsAssetMetadata[]>;
    subscribeToAccountBalances(account: string, assetsArray: Asset<Symbols>[], callback: (balances: PalletAssetsAssetAccount[]) => void): UnsubscribePromise;
}
