"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.init = void 0;
/* eslint-disable @typescript-eslint/no-use-before-define */
const xcm_config_1 = require("@moonbeam-network/xcm-config");
const lodash_1 = require("lodash");
const contracts_1 = require("../contracts");
const polkadot_1 = require("../polkadot");
const sdk_deposit_1 = require("./sdk.deposit");
const sdk_utils_1 = require("./sdk.utils");
const sdk_withdraw_1 = require("./sdk.withdraw");
function init(options) {
    return {
        moonbase: initByChain(xcm_config_1.moonbase, options),
        moonbeam: initByChain(xcm_config_1.moonbeam, options),
        moonriver: initByChain(xcm_config_1.moonriver, options),
    };
}
exports.init = init;
function initByChain(configBuilder, options) {
    const { symbols, assets, moonAsset, moonChain } = configBuilder;
    return {
        symbols,
        assets,
        moonAsset,
        moonChain,
        subscribeToAssetsBalanceInfo: async (account, cb) => {
            const [polkadot] = await (0, polkadot_1.createPolkadotServices)([
                configBuilder.moonChain.ws,
            ]);
            return (0, sdk_utils_1.subscribeToAssetsBalanceInfo)({
                account,
                polkadot,
                configBuilder,
                cb,
            });
        },
        deposit: (symbolOrAsset) => {
            const { chains, from } = configBuilder.deposit(symbolOrAsset);
            return {
                chains,
                from: (keyOrChain) => {
                    const { asset, origin, config } = from(keyOrChain);
                    return {
                        get: async (account, sourceAccount, { primaryAccount, polkadotSigner } = {}) => {
                            const signer = polkadotSigner || options?.polkadotSigner;
                            if ((0, lodash_1.isString)(sourceAccount) && !signer) {
                                throw new Error('Polkadot Signer/KeyringPair are not provided to XCM-SDK');
                            }
                            const [polkadot, foreignPolkadot] = await (0, polkadot_1.createPolkadotServices)([configBuilder.moonChain.ws, config.source.ws]);
                            const meta = foreignPolkadot.getMetadata();
                            const nativeAsset = configBuilder.assets[meta.symbol] ||
                                (0, sdk_utils_1.createDummyAsset)(meta.symbol);
                            return (0, sdk_deposit_1.getDepositData)({
                                account,
                                asset,
                                config,
                                foreignPolkadot,
                                moonChain,
                                nativeAsset,
                                origin,
                                polkadot,
                                polkadotSigner: signer,
                                primaryAccount,
                                sourceAccount,
                            });
                        },
                    };
                },
            };
        },
        withdraw: (symbolOrAsset) => {
            const { chains, to } = configBuilder.withdraw(symbolOrAsset);
            return {
                chains,
                to: (keyOrChain) => {
                    const { asset, origin, config } = to(keyOrChain);
                    return {
                        get: async (destinationAccount, { ethersSigner } = {}) => {
                            const signer = ethersSigner || options?.ethersSigner;
                            if (!signer) {
                                throw new Error('Ethers signer is not provided to XCM-SDK');
                            }
                            const contract = new contracts_1.XTokensContract(signer);
                            const [polkadot, destinationPolkadot] = await (0, polkadot_1.createPolkadotServices)([
                                configBuilder.moonChain.ws,
                                config.destination.ws,
                            ]);
                            const meta = destinationPolkadot.getMetadata();
                            const nativeAsset = configBuilder.assets[meta.symbol];
                            return (0, sdk_withdraw_1.getWithdrawData)({
                                asset,
                                config,
                                contract,
                                destinationAccount,
                                destinationPolkadot,
                                ethersSigner: signer,
                                moonChain,
                                nativeAsset,
                                origin,
                                polkadot,
                            });
                        },
                    };
                },
            };
        },
    };
}
//# sourceMappingURL=sdk.js.map