"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getMin = exports.getFee = exports.getWithdrawData = exports.createTransactionEventHandler = void 0;
const sdk_interfaces_1 = require("./sdk.interfaces");
async function createTransactionEventHandler(ethersSigner, txHash, cb, skipSentEvent = false) {
    if (!ethersSigner.provider) {
        throw new Error('options.ethersSigner has not provider');
    }
    if (!skipSentEvent) {
        cb({ status: sdk_interfaces_1.ExtrinsicStatus.Sent, txHash });
    }
    const tx = await ethersSigner.provider.getTransactionReceipt(txHash);
    if (!tx) {
        setTimeout(() => createTransactionEventHandler(ethersSigner, txHash, cb, true), 2000);
        return;
    }
    if (tx.status === 1) {
        cb({
            status: sdk_interfaces_1.ExtrinsicStatus.Success,
            blockHash: tx.blockHash,
            txHash,
        });
    }
    else {
        cb({
            status: sdk_interfaces_1.ExtrinsicStatus.Failed,
            blockHash: tx.blockHash,
            txHash,
        });
    }
}
exports.createTransactionEventHandler = createTransactionEventHandler;
async function getWithdrawData({ asset, config, contract, destinationAccount, destinationPolkadot, ethersSigner, moonChain, nativeAsset, origin, polkadot, }) {
    const meta = destinationPolkadot.getMetadata();
    const [decimals, destinationBalance, existentialDeposit, assetMinBalance] = await Promise.all([
        asset.isNative ? moonChain.decimals : polkadot.getAssetDecimals(asset),
        destinationPolkadot.getGenericBalance(destinationAccount, config.balance),
        asset.isNative ? 0n : destinationPolkadot.getExistentialDeposit(),
        config.sourceMinBalance
            ? destinationPolkadot.getAssetMinBalance(config.sourceMinBalance)
            : 0n,
    ]);
    const destinationFee = getFee(assetMinBalance, config);
    const min = getMin(assetMinBalance || existentialDeposit, destinationBalance, destinationFee);
    return {
        asset: { ...asset, decimals },
        destination: config.destination,
        destinationBalance,
        destinationFee,
        existentialDeposit,
        min,
        native: { ...nativeAsset, decimals: meta.decimals },
        origin,
        getFee: async (amount) => contract.getTransferFees(destinationAccount, amount, asset, config),
        send: async (amount, cb) => {
            const tx = await contract.transfer(destinationAccount, amount, asset, config);
            if (cb) {
                createTransactionEventHandler(ethersSigner, tx.hash, cb);
            }
            return tx.hash;
        },
    };
}
exports.getWithdrawData = getWithdrawData;
function getFee(assetMinBalance, config) {
    const calculatedFee = BigInt(config.weight * config.feePerWeight);
    return assetMinBalance > calculatedFee ? assetMinBalance : calculatedFee;
}
exports.getFee = getFee;
function getMin(balanceNeeded, destinationBalance, destinationFee) {
    const extra = destinationBalance >= balanceNeeded
        ? 0n
        : balanceNeeded - destinationBalance;
    return destinationFee + extra;
}
exports.getMin = getMin;
//# sourceMappingURL=sdk.withdraw.js.map